/**
 *  Satellit
 *
 *  Copyright (C) 2005  Yves Radunz, Volker Grabsch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import java.lang.*;
import javax.media.opengl.GL;
import com.sun.opengl.utils.GLUT;

public class Satellit
{
    public static final double gravconst=6.67259e-11;
    private double alarm;
    private double hoch;
    private double uzeit;
    private Vektor start;
    private Vektor richtung;

    /**
     * Neuen Satellit erstellen
     * Eingabe:
     * - zwei Punkte im Bogenmaß,
     * - die der Satellit überfliegt
     * - Masse bzw. Radius des Planeten
     * - Höhe bzw. Alarmschranke
     */
    public Satellit(double west1, double nord1, double west2, double nord2, double plmasse, double sathoch, double satalarm)
    {
        // erster Punkt
	start = Vektor.polar(1,west1,nord1);

        // zweiter Punkt
	Vektor sollrichtung=Vektor.polar(1,west2,nord2);

        // Zweiter Punkt, der überflogen wird, rechtwinklig zum ersten.
	richtung=Vektor.mul(Vektor.mul(start, sollrichtung),start);

	// Korrekt erzeugt?
	if (!start.iszero() && !richtung.iszero()){
	    // Richtung einstellen (Vorzeichen)
	    double winkel=Math.acos(start.prod(sollrichtung)/start.abs()/sollrichtung.abs());
	    if (winkel>Math.PI) richtung.neg();
	    richtung.skal(1/richtung.abs());
	    start.skal(1/start.abs());
	}else
	    throw new IllegalArgumentException();
	// Umlaufzeit berechnen
	uzeit = Math.sqrt(gravconst*plmasse/sathoch);
	// Flughöhe und Alarmschranke setzen
	hoch=sathoch; alarm=satalarm;
    }

    /**
     * Vektor start ausgeben
     */
    public Vektor getStart()
    {
	return start;
    }

    /**
     * Vektor richtung ausgeben
     */
    public Vektor getRichtung()
    {
	return richtung;
    }

    /**
     * Position des Satelliten
     */
    public Vektor getPosition(double zeit)
    {
        return Vektor.add(Vektor.skal(hoch*Math.sin(zeit/uzeit*2*Math.PI),richtung),Vektor.skal(hoch*Math.cos(zeit/uzeit*Math.PI*2),start));
    }

    /**
     * Abstand zwischen den Satelliten (zu einem bestimmten Zeitpunkt) => Für die Kommunikationsnetzbestimmung.
     */
    public double dist(Satellit sat, double zeit)
    {
	return Vektor.dist(sat.getPosition(zeit),getPosition(zeit));
    }

    /**
     * minimaler Abstand
     */
    public double mindist(Satellit sat)
    {
	double x1=0, x2=.2, xm=.1, uz=uzeit, dist1=dist(sat,0), dist2=dist(sat,uz/5), distm=dist(sat,uz/10);
	while(Math.abs(x1-x2)>=1e-10)
	    {
	    if (dist1<distm) {
		x2=xm;xm=x1;x1=2*x1-x2;
	    } else {
		if(dist2<distm) {
		    x1=xm;xm=x2;x2=2*x2-x1;
		} else {
		    if (dist1==distm && distm==dist2) 
			return dist1;
		    else {
			x1=(x1+xm)/2; x2=(x2+xm)/2;
		    }
		}
	    }
	    dist1=dist(sat,x1*uz);distm=dist(sat,xm*uz);dist2=dist(sat,x2*uz);
	}
	return distm;
    }

    /**
     * Kollisionsgefahr erkennen
     */
    public boolean kollgef(Satellit sat) 
    {
	return mindist(sat)<alarm;
    }

    /**
     * Umlaufzeit des Satelliten
     */
    public double getUzeit()
    {
	return uzeit;
    }

    /*****************************************************
     * Satelliten visualisieren
     *****************************************************/
    public void glVertex(GL gl, double zeit)
    {
	Vektor pos = getPosition(zeit);
	gl.glVertex3d(pos.x, pos.y, pos.z);
    }

    public void glTranslate(GL gl, double zeit)
    {
	Vektor pos = getPosition(zeit);
	gl.glTranslated(pos.x, pos.y, pos.z);
    }
}
